import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import QtQuick.Window 2.2
import U1db 1.0 as U1db
import Qt.labs.settings 1.0
import LocalStorageManager 1.0
import io.thp.pyotherside 1.4

/*!
    \brief MainView with a Label and Button elements.
*/

Window {
    id: main_window
    width: units.gu(100)
    height: units.gu(75)
    title: "uFTP"

    MainView {
        objectName: "mainView"

        // Note! applicationName needs to match the "name" field of the click manifest
        applicationName: "ftpmanager.costa"
        anchors.fill: parent
        anchorToKeyboard: true

        id: root
        /*
         This property enables the application to change orientation
         when the device is rotated. The default is false.
        */
        //automaticOrientation: true

        property var address;
        property var port;
        property var anonym;
        property var user;
        property var pwd;
        property var mlsd;
        property var move: null;
        property var type;
        property var size;

        Settings {}
        U1db.Database { id: db; path: "ftpmanager.u1db" }
            U1db.Index {
                database: db
                id: by_type
                /* You have to specify in the index all fields you want to retrieve
                   The query should return the whole document, not just indexed fields
                   https://bugs.launchpad.net/u1db-qt/+bug/1271973 */
                expression: ["name", "address", "port", "anonym", "user", "pwd", "mlsd", "sftp", "pka"]
            }
            U1db.Query {
                id: servers
                index: by_type
                query: ["*", "*"]
            }

            Python {
                id: py
                Component.onCompleted: {
                    // Add the directory of this .qml file to the search path
                    addImportPath(Qt.resolvedUrl('.'));

                    importModule('ftpmanager', function () {
                        console.log('imported python module');
                    });
                    importModule('sftpmanager', function () {
                        console.log('imported python module');
                    });
                }
            }

        PageStack { id:pageStack }

        LocalStorage {
            id: localStorage
        }

        Component.onCompleted: { pageStack.push(Qt.resolvedUrl("qml/Manage.qml")) }
    }
}
