import os
from ftplib import FTP, error_perm
from io import BytesIO
import pyotherside

class FtpTracker:
    sizeWritten = 0
    totalSize = 0
    lastShownPercent = 0

    def __init__(self, totalSize, qobj):
        self.totalSize = totalSize
        self.qobj = qobj

    def handle(self, block):
        self.sizeWritten += 1024
        percentComplete = round((self.sizeWritten / self.totalSize) * 100)

        if (self.lastShownPercent != percentComplete):
            self.lastShownPercent = percentComplete
            if percentComplete > 100:
                self.qobj.x = int(100)
            else:
                self.qobj.x = int(percentComplete)

class utils:
    def connect(self, mode, address, port, anonym, user=None, pwd=None):
        self.address = address
        self.anonym = anonym
        self.user = user
        self.pwd = pwd
        self.mode = mode
        self.error = 0
        try:
            self.ftp = FTP(timeout=5)
            if port == '':
                port = 21
            else:
                try:
                    port = int(port)
                except:
                    port = 21
            self.port = port
            self.ftp.connect(address, port)
            if anonym == True:
               self.ftp.login()
            else:
               self.ftp.login(user=user, passwd=pwd)
        except Exception as e:
            self.error = e

    def list(self):
        if(self.error == 0):
            listobj = []
            if self.mode == 'mlsd':
                try:
                    for i in self.ftp.mlsd():
                        if i[1]['type'] == 'file':
                            listobj.append({ 'name': str(i[0]), 'type': str('empty'), 'size': str(i[1]['size']) })
                        elif i[1]['type'] == 'dir':
                            listobj.append({ 'name': str(i[0]), 'type': str('folder'), 'size': str('FOLDER') })
                    return listobj
                except error_perm:
                    return str('mlsderror')
                except:
                    return str('connerror')
            else:
                listitem = self.ftp.nlst()
                for item in listitem:
                    listobj.append({ 'name': str(item), 'type': str(self.isfile(item)), 'size': str(self.getsize(item)) })
                return listobj
        else:
              return [ { 'name': str(self.error), 'type': 'error', 'size': str('none') } ]

    def chdir(self, dir):
        try:
            return self.ftp.cwd(dir)
        except Exception as e:
            return str('error')

    def download(self, name, path, size, qobj):
        def callback(data):
            try:
                downloadTracker.handle(data)
                file.write(data)
            except:
                return None
        try:
            self.reconn(self.address, self.port, self.anonym, self.user, self.pwd)
            self.rftp.cwd(path)
            file = open(os.path.expanduser('~/.local/share/ftpmanager.costa/' + name), 'wb')
            downloadTracker = FtpTracker(int(size), qobj)
            self.rftp.retrbinary('RETR ' + name, callback, 1024)
            self.rftp.sock.close()
        except:
            return str('error')

    def upload(self, name, path, qobj):
        try:
            self.reconn(self.address, self.port, self.anonym, self.user, self.pwd)
            self.rftp.cwd(path)
            qobj.n = 0
            for items in name:
                size = os.path.getsize(items)
                uploadTracker = FtpTracker(int(size), qobj)
                self.rftp.storbinary('STOR ' + items.rsplit('/',1)[1], open(items, 'rb'), 1024, uploadTracker.handle)
                qobj.n = qobj.n + 1
            self.rftp.sock.close()
            return None
        except:
            return str('error')

    def newdir(self, name):
        try:
            self.ftp.mkd(name)
        except:
            return str('error')

    def deletefile(self, name):
        try:
            self.ftp.delete(name)
        except:
            return str('error')

    def deletedir(self,start, current, ent=0, ex=0):
        self.deleteError = False
        i = 0
        try:
            self.ftp.cwd(current)
        except:
            self.deleteError = True
            return str('error')
        listitem = self.ftp.nlst()
        for item in listitem:
            try:
                self.ftp.rmd(item)
            except:
                try:
                    self.ftp.delete(item)
                except:
                    i = 1
                    self.deletedir(start=start, current=item, ent=ent+1)
        if i == 0 and ex < ent:
            ex=ex+1
            self.deletedir(start=start, current='..')
        if i == 0 and ex == ent:
            self.ftp.cwd('..')
            self.ftp.rmd(start)
        if self.deleteError == True:
            while ex == ent:
                ex=ex+1
                self.ftp.cwd('..')
            return str('error')

    def isfile(self, name):
        try:
           self.ftp.sendcmd('MDTM ' + name)
        except:
            return 'folder'
        else:
            try:
                self.ftp.size(name)
                return 'empty'
            except:
                return 'folder'

    def getsize(self, name):
        try:
            return self.ftp.size(name)
        except:
            return 'FOLDER'
        
    def rename(self, start, end):
        try:
            self.ftp.rename(start, end)
        except:
            return str('error')

    def opentext(self, name):
        try:
            self.reconn(self.address, self.port, self.anonym, self.user, self.pwd, 10)
            self.rftp.cwd(self.path())
            data = []
            def handle_binary(more_data):
                data.append(more_data)
            resp = self.rftp.retrbinary('RETR ' + name, callback=handle_binary)
            data = b''.join(data)
            self.rftp.sock.close()
            return data
        except:
            return str('error')

    def savetext(self, name, content):
        try:
            self.reconn(self.address, self.port, self.anonym, self.user, self.pwd, 10)
            self.rftp.cwd(self.path())
            self.rftp.storbinary('STOR ' + name, BytesIO(bytes(content, 'UTF-8')))
            self.rftp.sock.close()
            return None
        except:
            return str('error')

    def path(self):
        try:
            return str(self.ftp.pwd())
        except:
            return str('*error')

    def reconn(self, address, port, anonym, user=None, pwd=None, out=500):
        try:
            self.rftp = FTP(timeout=out)
            self.rftp.connect(address, port)
            if anonym == True:
               self.rftp.login()
            else:
               self.rftp.login(user=user, passwd=pwd)
        except Exception as e:
            return str('error')

    def exit(self):
        if(self.error == 0):
            try:
                self.ftp.quit()
                return None
            except:
                return None

utils = utils()
