import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import Qt.labs.folderlistmodel 2.1
import Ubuntu.Content 1.3
import "FileExtensionMapper.js" as JSType

Page {
    id: downPage
    title: i18n.tr("Downloads")
    head.backAction: Action {
        iconName: "back"
        onTriggered: {
            pageStack.pop(downPage)
        }
    }
    property string fileurl
    ServerListView {
        anchors.fill: parent
        FolderListModel {
            id: folderModel
            folder : "file://" + localStorage.getHomePath() + "/.local/share/ftpmanager.costa"
            nameFilters: ["*"]
            showDirs: false
            showHidden : true
        }
        Component {
            id: fileDelegate
            ListItem {
                id: lstItem
                Icon {
                    anchors {
                       left: parent.left
                       leftMargin: units.gu(3)
                       verticalCenter: parent.verticalCenter
                    }
                    name: if(fileIsDir === true){ "folder" }else{ localStorage.iconNameForFile(filePath) }
                    height: units.gu(6)
                    width: units.gu(6)
                }
                Item {
                    anchors {
                       left: parent.left
                       top: parent.top
                       leftMargin: units.gu(11)
                       topMargin: units.gu(1)
                    }
                    Column {
                        Label {
                            text: fileName
                            font.pixelSize: units.gu(2)
                            font.weight: Font.Light
                            horizontalAlignment: Text.AlignRight
                        }
                        Label {
                            text: downPage.getSize(fileSize)
                            font.pixelSize: units.gu(2)
                            font.weight: Font.Light
                            horizontalAlignment: Text.AlignRight
                            color: "#BDBDBD"
                        }
                    }
                }
                onClicked: {
                    if(JSType.filenameToContentType(fileName, "edit")){
                        chooser.show()
                        chooser.path = filePath
                        chooser.name = fileName
                        chooser.size = downPage.getSize(fileSize)
                    }else{
                        if(fileName === "ftpmanager.u1db")
                            pageStack.push(pick, {url: "config"})
                        else
                            pageStack.push(pick, {url: filePath})
                    }
                }

                leadingActions: ListItemActions {
                    actions: [
                        Action {
                            text: i18n.tr("Delete")
                            iconName: "delete"
                            onTriggered: {
                                delDialog.show()
                                delDialog.filePath = filePath
                            }
                        }
                    ]
                }
                trailingActions: ListItemActions {
                    actions: [
                        Action {
                            text: i18n.tr("Rename")
                            iconName: "edit"
                            onTriggered: {
                                modDialog.show()
                                tx.text = fileName
                                modDialog.fileName = fileName
                                modDialog.filePath = filePath
                            }
                        }
                    ]
                }
            }
        }
        model: folderModel
        delegate: fileDelegate
    }
    function getSize(size){
        var factor
        var type
        if (size < Math.pow(2,10)){
            factor = 1
            type = 'bytes'
        }else if(size < Math.pow(2,20)){
            factor = Math.pow(2,10)
            type = 'KB'
        }else if(size < Math.pow(2,30)){
            factor = Math.pow(2,20)
            type = 'MB'
        }else if(size < Math.pow(2,40)){
            factor = Math.pow(2,30)
            type = 'GB'
        }
        return Math.round(size/factor) + type
    }
    Dialog {
        id: chooser
        property string path;
        property string name;
        property string size;
        Column {
            spacing: units.gu(2)

            Item {
                width: parent.width
                height: units.gu(5)

                Icon {
                    id: mimetypeIcon
                    name: localStorage.iconNameForFile(chooser.path)
                    height: units.gu(5.5)
                    width: height
                }

                Label {
                    id: filenameLabel
                    anchors.top: mimetypeIcon.top
                    anchors.topMargin: units.gu(0.5)
                    anchors.left: mimetypeIcon.right
                    anchors.leftMargin: units.gu(2)
                    anchors.right: parent.right
                    anchors.rightMargin: units.gu(2)
                    elide: Text.ElideMiddle
                    text: chooser.name
                }

                Label {
                    anchors.top: filenameLabel.bottom
                    anchors.left: filenameLabel.left
                    anchors.right: filenameLabel.right
                    elide: Text.ElideRight
                    font.capitalization: Font.Capitalize
                    text: chooser.size
                }
            }

            Button {
                text: i18n.tr("Edit")
                anchors.horizontalCenter: parent.horizontalCenter
                width: units.gu(22)
                height: units.gu(4)
                onClicked: {
                    pageStack.push(Qt.resolvedUrl("EditFile.qml"), {name: chooser.name, path: chooser.path, local: true})
                    chooser.hide()
                }
            }

            Button {
                text: i18n.tr("Open with other app")
                anchors.horizontalCenter: parent.horizontalCenter
                width: units.gu(22)
                height: units.gu(4)
                onClicked: {
                    if(chooser.name === "ftpmanager.u1db")
                        pageStack.push(pick, {url: "config"})
                    else
                        pageStack.push(pick, {url: chooser.path})
                    chooser.hide()
                }
            }

            Button {
                text: i18n.tr("Cancel")
                anchors.horizontalCenter: parent.horizontalCenter
                width: units.gu(22)
                height: units.gu(4)
                onClicked: chooser.hide()
            }
        }
    }

    Dialog {
        id: delDialog
        title: i18n.tr("Delete file?")
        property string filePath;
        Button {
            id: delBtn
            text: i18n.tr("OK")
            color: UbuntuColors.orange
            onClicked: {
                delBtn.enabled = false
                cancBtn.enabled = false
                if(localStorage.remove(delDialog.filePath) === false){
                    error.show()
                    error.title = i18n.tr("Unable to erase the file")
                }
                delBtn.enabled = true
                cancBtn.enabled = true
                delDialog.hide()
            }
        }
        Button {
            id: cancBtn
            text: i18n.tr("Cancel")
            onClicked: {
                delDialog.hide()
            }
        }
    }
    Dialog {
        id: modDialog
        property string filePath;
        property string fileName;
        title: i18n.tr("Rename")
        Label {
            text: i18n.tr("Write the new name:")
        }

        TextField {
            id: tx
            text: modDialog.fileName
            inputMethodHints: Qt.ImhNoPredictiveText
        }

        Button {
            text: i18n.tr("OK")
            color: UbuntuColors.blue
            onClicked: {
                modDialog.hide()
                if(tx.text === modDialog.fileName){ }
                else if(localStorage.rename(modDialog.filePath, localStorage.getHomePath() + "/.local/share/ftpmanager.costa/" + tx.text) === false){
                    error.show()
                    error.title = i18n.tr("Unable to rename the file")
                }
            }
        }
        Button {
            text: i18n.tr("Cancel")
            onClicked: modDialog.hide()
        }
    }
    Dialog {
        id: error
        Button {
            text: i18n.tr("OK")
            color: UbuntuColors.orange
            onClicked: {
                error.hide()
            }
        }
    }
    Component {
        id: pick
        Page {
            id: picker
            title: i18n.tr("Choose")

            Component {
                id: resultComponent
                ContentItem {}
            }

            visible: true
            property string url
            property var curTransfer

            function __exportItems(url) {
                if (picker.curTransfer.state === ContentTransfer.InProgress)
                {
                    picker.curTransfer.items = [ resultComponent.createObject(picker, {"url": url}) ];
                    picker.curTransfer.state = ContentTransfer.Charged;
                }
            }

            ContentPeerPicker {
                anchors.fill: parent
                visible: parent.visible
                contentType: if(picker.url === "config"){ ContentType.Unknown }else{ ContentType.All }
                handler: ContentHandler.Destination
                showTitle: false
                onPeerSelected: {
                    picker.curTransfer = peer.request();
                    pageStack.removePages(picker);
                    if (picker.curTransfer.state === ContentTransfer.InProgress)
                    picker.__exportItems(picker.url);
                }
                onCancelPressed: {
                    pageStack.removePages(picker);
                }
            }

            Connections {
                target: picker.curTransfer
                onStateChanged: {
                    console.log("curTransfer StateChanged: " + picker.curTransfer.state);
                    if (picker.curTransfer.state === ContentTransfer.InProgress) {
                        picker.__exportItems(picker.url);
                    }
                }
            }
        }
    }
}
