import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.ListItems 1.3 as ListItem
import Ubuntu.Components.Popups 1.3
import "Global.js" as Utils

Page {
    id: editPage
    title: i18n.tr("Modify server")

    head.actions: [
            Action {
                id: newAction
                parameterType: Action.String
                text: i18n.tr("Save")
                iconName: "ok"
                enabled: canSave
                onTriggered: {
                    editPage.adddoc()
                    pageStack.pop(editPage)
                }
            }

        ]

    head.backAction: Action {
        iconName: "back"
        onTriggered: {
            if(serverObj.anonym === true) {
                if(name.text !== serverObj.name || address.text !== serverObj.address || anonym.checked !== serverObj.anonym || mlsdSwitch.checked !== serverObj.mlsd){
                    backDialog.show()
                }else{
                    pageStack.pop(editPage)
                }
            }
            if(serverObj.anonym === false) {
                if(name.text !== serverObj.name || address.text !== serverObj.address || anonym.checked !== serverObj.anonym || user.text !== serverObj.user || pwd.text !== serverObj.pwd || mlsdSwitch.checked !== serverObj.mlsd){
                    backDialog.show()
                }else{
                    pageStack.pop(editPage)
                }
            }
        }
    }

    property var serverObj: db.getDoc(Utils.cached)

    property bool sftp: serverObj.sftp;
    property bool canSave: (name.text !== "") && (address.text !== "")

    function adddoc() {
        if(anonym.checked === true) {
            db.putDoc({name: name.text, address: address.text, port: port.text, anonym: true, mlsd: mlsdSwitch.checked, sftp: sftp, pka: pka.checked}, Utils.cached)
        }else{
            db.putDoc({name: name.text, address: address.text, port: port.text, anonym: false, user: user.text, pwd: pwd.text, mlsd: mlsdSwitch.checked, sftp: sftp, pka: pka.checked}, Utils.cached)
        }
    }

    Dialog {
        id: backDialog
        title: i18n.tr("Do you want to erase all changes?")
        Button {
            text: i18n.tr("OK")
            color: UbuntuColors.orange
            onClicked: {
                pageStack.pop(editPage)
                backDialog.hide()

            }
        }

        Button {
            text: i18n.tr("Save")
            enabled: canSave
            onClicked: {
                editPage.adddoc()
                pageStack.pop(editPage);
                backDialog.hide()
            }
        }
        Button {
            text: i18n.tr("Cancel")
            onClicked: backDialog.hide()
        }
    }

    Flickable {
        id: content
        anchors.fill: parent
        contentHeight: units.gu(62)
        Column {
            spacing: units.gu(1)
            anchors {
                margins: units.gu(2)
                fill: parent
            }

            Label {
                text: i18n.tr("Server name")
            }

            TextField {
                id: name
                anchors.left: parent.left
                anchors.right: parent.right
                inputMethodHints: Qt.ImhNoPredictiveText
            }
            Label {
                text: i18n.tr("Server address")
            }
            TextField {
                id: address
                anchors.left: parent.left
                anchors.right: parent.right
                inputMethodHints: Qt.ImhNoPredictiveText
            }
            ListItem.Standard {
                text: i18n.tr("Port")
                control: TextField {
                    id: port
                    width: units.gu(10)
                    placeholderText : if(sftp){"22"}else{"21"}
                    horizontalAlignment: TextInput.AlignHCenter
                    hasClearButton: false
                    validator: IntValidator{}
                    inputMethodHints: Qt.ImhDigitsOnly
                }
            }
            ListItem.Standard {
                text: i18n.tr("Anonymous user")
                visible: !sftp
                control: Switch {
                    id: anonym
                    anchors.verticalCenter: parent.verticalCenter
                    checked: false
                    onCheckedChanged: {
                        if (anonym.checked === true){
                            user.enabled = false;
                            pwd.enabled = false;
                            show.enabled = false;
                        }else{
                            user.enabled = true;
                            pwd.enabled = true;
                            show.enabled = true;
                        }
                    }
                  }
            }
            ListItem.Standard {
                text: i18n.tr("Key-based authentication (RSA)")
                visible: sftp
                control: Switch {
                    id: pka
                    anchors.verticalCenter: parent.verticalCenter
                    checked: false
                    onCheckedChanged: {
                        if (pka.checked === true){
                            pwd.enabled = false;
                            show.enabled = false;
                            pwd.text = ""
                            pwd.placeholderText = i18n.tr("Reading key from ~/.config/ftpmanager.costa/id_rsa")
                        }else{
                            pwd.enabled = true;
                            show.enabled = true;
                            pwd.text = ""
                            pwd.placeholderText = ""
                        }
                    }
                  }
            }
            Label {
                text: i18n.tr("User name")
            }
            TextField {
                id: user
                anchors.left: parent.left
                anchors.right: parent.right
                inputMethodHints: Qt.ImhNoPredictiveText
            }
            Label {
                text: i18n.tr("Password")
            }
            TextField {
                id: pwd
                anchors.left: parent.left
                anchors.right: parent.right
                echoMode: TextInput.Password
            }
            ListItem.Standard {
                text: i18n.tr("Show password")
                control: Switch {
                    id: show
                    anchors.verticalCenter: parent.verticalCenter
                    checked: false
                    onCheckedChanged: {
                        if (show.checked === true){
                            pwd.echoMode = TextInput.Normal;
                            pwd.inputMethodHints = Qt.ImhNoPredictiveText;
                        }else{
                            pwd.echoMode = TextInput.Password;
                        }
                    }
                  }
            }
            ListItem.Standard {
                text: i18n.tr("Enable RFC 3659 protocol")
                visible: !sftp
                control: Switch {
                    id: mlsdSwitch
                    anchors.verticalCenter: parent.verticalCenter
                    checked: serverObj.mlsd
                  }
            }

            function load() {
                if(serverObj.anonym === true) {
                    name.text = serverObj.name
                    address.text = serverObj.address
                    anonym.checked = true
                    port.text = serverObj.port
                    pka.checked = serverObj.pka
                }else{
                    name.text = serverObj.name
                    address.text = serverObj.address
                    anonym.checked = false
                    user.text = serverObj.user
                    pwd.text = serverObj.pwd
                    port.text = serverObj.port
                    pka.checked = serverObj.pka
                }
            }

            Component.onCompleted: load()

        }
    }
}
