import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3


Page {
    id: textPage
    property string path;
    property string name;
    property bool local; //valore per deterinare se usare Python o C++
    property string ftptype: "ftpmanager"
    title: name

    head.actions: [
        Action {
            id: saveAction
            text: i18n.tr("Save")
            parameterType: Action.String
            iconName: "ok"
            visible: false
            onTriggered: {
                saveAction.enabled = false
                if(local){
                    if(localStorage.write(textPage.path, textAreaMain.text) === false){
                        error.show()
                    }else{
                        pageStack.pop(textPage)
                    }
                }else{
                    py.call(ftptype + '.utils.savetext', [textPage.name, textAreaMain.text], function(result) {
                        if(result === "error"){
                            error.show()
                            textAreaMain.readOnly = true
                        }else{
                            pageStack.pop(textPage)
                        }
                    });
                }
            }
        },
        Action {
            id: editAction
            text: i18n.tr("Edit")
            iconName: "edit"
            enabled: false
            onTriggered: {
                editAction.visible = false
                textAreaMain.readOnly = false
                saveAction.visible = true
            }
        }

    ]

    TextArea {
        id: textAreaMain
        objectName: "textAreaMain"

        property bool isLoading: true

        anchors.fill: parent

        // FIXME: If set to true, some of the keyboard hooks are disabled
        // And it's not possible to move the cursor with arrow keys.
        readOnly: true

        text: i18n.tr("Loading...")
        font.family: "UbuntuMono"

        inputMethodHints: Qt.ImhNoPredictiveText

        Component.onCompleted: {
            if(local){
                textAreaMain.text = localStorage.read(textPage.path)
                editAction.enabled = true
            }else{
                py.call(ftptype + '.utils.opentext', [textPage.name], function(result) {
                    if(result === "error"){
                        textAreaMain.text = i18n.tr("Unable to open file")
                    }else{
                        textAreaMain.text = result
                        editAction.enabled = true
                    }
                });
            }
        }
    }

    Dialog {
        id: error
        title: i18n.tr("Unable to save the file")
        Button {
            text: i18n.tr("OK")
            color: UbuntuColors.orange
            onClicked: {
                error.hide()
            }
        }
    }
}
