import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import Ubuntu.Content 1.3
import "Connect.js" as Connect
import "FileExtensionMapper.js" as JSType

Page {
    id: explorePage
    title: i18n.tr("Explore")

    property bool exit;
    property var path;
    property string name;
    property string indsize;
    property string type;

    property var activeTransfer;

    property string ftptype: "ftpmanager";

    property bool back: true;

    head.backAction: Action {
        iconName: "back"
        onTriggered: {
            if(explorePage.back === false) {
                explorePage.back = true
                noFile.visible = false
                list.visible = true
                newAction.enabled = true
                pasteAction.enabled = true
                upload.enabled = true
            }
            else if(exit === true) {
                py.call(ftptype + '.utils.exit')
                main_window.title = "uFTP"
                pageStack.pop(explorePage)
            }
            else {
                py.call(ftptype + '.utils.chdir', ['..'], function() {})
                pageStack.pop(explorePage)
            }
        }
    }
    head.actions: [
        Action {
            id: newAction
            parameterType: Action.String
            text: i18n.tr("New folder")
            iconName: "add"
            enabled: false
            onTriggered: newDir.show()
        },
        Action {
            id: pasteAction
            parameterType: Action.String
            text: i18n.tr("Paste")
            iconName: "edit-paste"
            enabled: false
            visible: root.move !== null
            onTriggered: {
                var parts = root.move.split('/');
                var file = parts[parts.length - 1];
                py.call(ftptype + '.utils.rename', [root.move, explorePage.path + "/" + file], function(result) {
                    if(result === "error"){
                        error.show()
                        error.title = i18n.tr("Unable to move object")
                    }else{
                        listModel.append({"name": file, "type": root.type, "size": root.size})
                        noFile.visible = false
                        root.size = "success"
                    }
                    root.move = null
                });
            }
            onVisibleChanged: {
                if(root.size === "success"){
                    explorePage.removeItem()
                }
            }
        },
        Action {
            id: upload
            parameterType: Action.String
            text: i18n.tr("Upload")
            iconSource: "../img/up.svg"
            enabled: false
            onTriggered: PopupUtils.open(contentpop)
        },
        Action {
            id: reload
            parameterType: Action.String
            text: i18n.tr("Reload")
            iconName: "reload"
            visible: false
            onTriggered: {
                noFile.visible = false
                reload.visible = false
                activity.visible = true
                activity.running = true
                newAction.enabled = false
                pasteAction.enabled = false
                upload.enabled = false
                if(ftptype === "sftpmanager")
                    Connect.conn(true, root.mlsd)
                else
                    Connect.conn(false, root.mlsd)
                py.call(ftptype + '.utils.chdir', [explorePage.path], function(result) {
                    if(result === 'error'){
                        activity.visible = false
                        noFile.visible = true
                        noFile.text = i18n.tr("Unable to access to the folder")
                    }else{
                        explorePage.update()
                    }
                });
            }
        }

    ]

    Loader {
        id: view
        anchors {
            fill: parent
        }
    }


    Label {
        id:noFile
        anchors.centerIn: parent
        text: i18n.tr("Empty folder")
        visible: false
    }

    ActivityIndicator {
        id: activity
        anchors.centerIn: parent
        running: true
    }

    ServerListView {
        id: list
        anchors.fill: parent

        model: ListModel {
            id: listModel
            Component.onCompleted: {
                py.call(ftptype + '.utils.list', [], function(result) {
                    if(result === "mlsderror"){
                        activity.visible = false
                        noFile.text = i18n.tr("The server doesn't support RFC 3659 protocol.\n Please disable it.")
                        noFile.visible = true
                    }
                    else if(result === "connerror"){
                        activity.visible = false
                        noFile.text = i18n.tr("Connection lost. Refresh page")
                        noFile.visible = true
                        reload.visible = true
                    }
                    else{
                        for (var i=0; i<result.length; i++) {
                            listModel.append(result[i]);
                        }
                        if(result.length === 0){
                            activity.visible = false
                            noFile.visible = true
                        }
                        newAction.enabled = true
                        pasteAction.enabled = true
                        upload.enabled = true
                    }
                });
            }
        }

        delegate: ListItem {
            id: lstItem
            Icon {
                anchors {
                   left: parent.left
                   leftMargin: units.gu(3)
                   verticalCenter: parent.verticalCenter
                }
                name: lstItem.gettype()
                height: units.gu(6)
                width: units.gu(6)
            }
            Item {
                anchors {
                   left: parent.left
                   top: parent.top
                   leftMargin: units.gu(11)
                   topMargin: units.gu(1)
                }
                Column {
                    Label {
                        text: name
                        font.pixelSize: units.gu(2)
                        font.weight: Font.Light
                        horizontalAlignment: Text.AlignRight
                    }
                    Label {
                        text: if(size !== "FOLDER") { explorePage.getSize(size) }else{ "FOLDER" }
                        font.pixelSize: units.gu(2)
                        font.weight: Font.Light
                        horizontalAlignment: Text.AlignRight
                        color: "#BDBDBD"
                    }
                }
            }
            onClicked: {
                if (type == 'folder') {
                    list.visible = false
                    activity.visible = true
                    activity.running = true
                    newAction.enabled = false
                    pasteAction.enabled = false
                    upload.enabled = false
                    py.call(ftptype + '.utils.chdir', [name], function(result) {
                        if(result === 'error'){
                            activity.visible = false
                            noFile.visible = true
                            explorePage.back = false
                            noFile.text = i18n.tr("Unable to access to the folder")
                        }else{
                            newAction.enabled = true
                            pasteAction.enabled = true
                            upload.enabled = true
                            activity.visible = false
                            list.visible = true
                            pageStack.push(Qt.resolvedUrl("Explore.qml"), {exit: false, path: explorePage.path + "/" + name, ftptype: ftptype})
                        }
                    });
                }else{
                    explorePage.name = name
                    explorePage.indsize = size
                    downloadDialog.show()
                }
            }

            function move(){
                for (var i=0; i<listModel.count; ++i) {
                    listModel.setProperty(i,"selected",false)
                }
                root.move = explorePage.path + "/" + name
                root.type = type
                root.size = size
                listModel.setProperty(index,"selected",!selected)
            }
            function gettype(){
                if(type !== "folder"){
                    return localStorage.iconNameForFileName(name)
                }else{
                    return "folder"
                }
            }
            function getname(){
                explorePage.indsize = index
                return name
            }

            leadingActions: ListItemActions {
                actions: [
                    Action {
                        text: i18n.tr("Delete")
                        iconName: "delete"
                        onTriggered: {
                            explorePage.name = lstItem.getname()
                            explorePage.type = lstItem.gettype()
                            delDialog.show()
                        }
                    }
                ]
            }
            trailingActions: ListItemActions {
                actions: [
                    Action {
                        text: i18n.tr("Rename")
                        iconName: "edit"
                        onTriggered: {
                            explorePage.name = lstItem.getname()
                            modDialog.show()
                            modDialog.title = i18n.tr("Rename")
                        }
                    },
                    Action {
                        text: i18n.tr("Cut")
                        iconName: "edit-cut"
                        onTriggered: {
                            lstItem.move()
                        }
                    }
                ]
            }
            Component.onCompleted: {
                if(type === 'error'){
                    explorePage.type = name
                    list.visible = false
                    view.sourceComponent = errorSection
                }
                activity.running = false
            }
        }
    }
    QtObject {
        id: qo
        property int x: 0;
        property int n: 0;
    }
    Dialog {
        id: delDialog
        title: if(explorePage.type === "folder")
                   i18n.tr("Delete folder and its contents?")
               else
                   i18n.tr("Delete file?")
        Button {
            id: delBtn
            text: i18n.tr("OK")
            color: UbuntuColors.orange
            onClicked: {
                delBtn.enabled = false
                cancBtn.enabled = false
                if(explorePage.type === "folder"){
                    py.call(ftptype + '.utils.deletedir', [explorePage.name, explorePage.name], function(result) {
                        if(result === "error"){
                            error.show()
                            error.title = i18n.tr("Unable to erase the folder")
                        }else{
                            listModel.remove(explorePage.indsize);
                            if(listModel.count === 0){
                                noFile.text = i18n.tr("Empty folder")
                                noFile.visible = true
                            }
                        }
                        delDialog.hide()
                        delBtn.enabled = true
                        cancBtn.enabled = true
                    });
                }else{
                    py.call(ftptype + '.utils.deletefile', [explorePage.name], function(result) {
                        if(result === "error"){
                            error.show()
                            error.title = i18n.tr("Unable to erase the file")
                        }else{
                            listModel.remove(explorePage.indsize);
                            if(listModel.count === 0){
                                noFile.text = i18n.tr("Empty folder")
                                noFile.visible = true
                            }
                        }
                        delDialog.hide()
                        delBtn.enabled = true
                        cancBtn.enabled = true
                    });
                }
            }
        }
        Button {
            id: cancBtn
            text: i18n.tr("Cancel")
            onClicked: {
                delDialog.hide()
            }
        }
    }
    Dialog {
        id: modDialog
        Label {
            text: i18n.tr("Write the new name:")
        }

        TextField {
            id: tx
            text: explorePage.name
            inputMethodHints: Qt.ImhNoPredictiveText
        }

        Button {
            text: i18n.tr("OK")
            color: UbuntuColors.blue
            onClicked: {
                py.call(ftptype + '.utils.rename', [explorePage.name, tx.text], function(result) {
                    if(result === "error"){
                        error.show()
                        error.title = i18n.tr("Unable to rename object")
                    }else{
                        listModel.setProperty(explorePage.indsize, "name", tx.text);
                    }
                });
                modDialog.hide()
            }
        }
        Button {
            text: i18n.tr("Cancel")
            onClicked: modDialog.hide()
        }
    }
    Dialog {
        id: downloadDialog
        Column {
            spacing: units.gu(2)

            Item {
                width: parent.width
                height: units.gu(5)

                Icon {
                    id: mimetypeIcon
                    name: localStorage.iconNameForFileName(explorePage.name)
                    height: units.gu(5.5)
                    width: height
                }

                Label {
                    id: filenameLabel
                    anchors.top: mimetypeIcon.top
                    anchors.topMargin: units.gu(0.5)
                    anchors.left: mimetypeIcon.right
                    anchors.leftMargin: units.gu(2)
                    anchors.right: parent.right
                    anchors.rightMargin: units.gu(2)
                    elide: Text.ElideMiddle
                    text: explorePage.name
                }

                Label {
                    anchors.top: filenameLabel.bottom
                    anchors.left: filenameLabel.left
                    anchors.right: filenameLabel.right
                    elide: Text.ElideRight
                    font.capitalization: Font.Capitalize
                    text: explorePage.getSize(explorePage.indsize)
                }
            }

            Button {
                text: i18n.tr("Download")
                anchors.horizontalCenter: parent.horizontalCenter
                width: units.gu(22)
                height: units.gu(4)
                onClicked: {
                    qo.x = 0
                    downloadDialog.hide()
                    downloading.show()
                    downloading.title = i18n.tr("Downloading...")
                    py.call(ftptype + '.utils.download', [explorePage.name, explorePage.path, explorePage.indsize, qo], function(result) {
                        downloading.hide()
                        if(result === "error"){
                            error.show()
                            error.title = i18n.tr("Unable to download the file")
                        }
                    });
                }
            }

            Button {
                text: i18n.tr("Edit")
                anchors.horizontalCenter: parent.horizontalCenter
                width: units.gu(22)
                height: units.gu(4)
                visible: explorePage.indsize < 1000000 && JSType.filenameToContentType(explorePage.name, "edit")
                onClicked: {
                    pageStack.push(Qt.resolvedUrl("EditFile.qml"), {name: explorePage.name, local: false, ftptype: ftptype})
                    downloadDialog.hide()
                }
            }

            Button {
                text: i18n.tr("Cancel")
                anchors.horizontalCenter: parent.horizontalCenter
                width: units.gu(22)
                height: units.gu(4)
                onClicked: downloadDialog.hide()
            }
        }
    }

    Component {
        id: errorSection
        Column {
            anchors.fill: parent
            anchors.margins: units.gu(4)
            anchors.topMargin: units.gu(10)

            spacing: units.gu(3)

            Label {
                width: parent.width
                fontSize: "x-large"
                text: i18n.tr("Network error")
            }

            Label {
                width: parent.width
                text: i18n.tr("It seems there are errors loading the page.")
                wrapMode: Text.Wrap
            }

            Label {
                width: parent.width
                text: i18n.tr("Try to verify connection and credentials, then refresh the page.")
                wrapMode: Text.Wrap
            }
            Label {
                width: parent.width
                text: i18n.tr("Error string: ") + explorePage.type
                wrapMode: Text.Wrap
            }

            Button {
                text: i18n.tr("Refresh page")
                onClicked: {
                    pageStack.pop(explorePage)
                    if(ftptype === "sftpmanager")
                        Connect.conn(true, root.mlsd)
                    else
                        Connect.conn(false, root.mlsd)
                    pageStack.push(Qt.resolvedUrl("Explore.qml"), {exit: true, ftptype: ftptype})
                }
            }
            Component.onCompleted: {
                newAction.enabled = false
                pasteAction.enabled = false
                upload.enabled = false
            }
        }
    }

    Dialog {
        id: error
        Button {
            text: i18n.tr("OK")
            color: UbuntuColors.orange
            onClicked: {
                error.hide()
                list.enabled = true
            }
        }
    }
    Dialog {
        id: downloading
        property int tot;
        ProgressBar {
            id: files
            visible: false
            value: qo.n
            minimumValue: 0
            maximumValue: downloading.tot
        }
        ProgressBar {
            id: progress
            value: qo.x
            minimumValue: 0
            maximumValue: 100
        }
    }
    Dialog {
        id: newDir
        title: i18n.tr("New folder")
        Label {
            text: i18n.tr("Write the name of the folder:")
        }

        TextField {
            id: textDir
            inputMethodHints: Qt.ImhNoPredictiveText
        }

        Button {
            text: i18n.tr("OK")
            color: UbuntuColors.green
            onClicked: {
                py.call(ftptype + '.utils.newdir', [textDir.text], function(result) {
                    if(result === "error"){
                        error.show()
                        error.title = i18n.tr("Unable to create the folder")
                    }else{
                        explorePage.update()
                    }
                });
                textDir.text = ""
                newDir.hide()
            }
        }
        Button {
            text: i18n.tr("Cancel")
            onClicked: {
                textDir.text = ""
                newDir.hide()
            }
        }
    }

    Component.onCompleted: {
        py.call(ftptype + '.utils.path', [], function(result) {
            if(result !== "*error")
                explorePage.path = result
        });
    }

    function update(){
        pageStack.pop(explorePage)
        if(exit === true){
            pageStack.push(Qt.resolvedUrl("Explore.qml"), {exit: true, ftptype: ftptype})
        }else{
            pageStack.push(Qt.resolvedUrl("Explore.qml"), {exit: false, ftptype: ftptype})
        }
    }

    function removeItem(){ /* listModel.setProperty(index,"selected",!selected) */
        for (var i=0; i<listModel.count; ++i) {
            if (listModel.get(i).selected){
                listModel.remove(i);
                i=0;
            }
        }
    }

    function getSize(size){
        var factor
        var type
        if (size < Math.pow(2,10)){
            factor = 1
            type = 'bytes'
        }else if(size < Math.pow(2,20)){
            factor = Math.pow(2,10)
            type = 'KB'
        }else if(size < Math.pow(2,30)){
            factor = Math.pow(2,20)
            type = 'MB'
        }else if(size < Math.pow(2,40)){
            factor = Math.pow(2,30)
            type = 'GB'
        }
        return Math.round(size/factor) + type
    }

    Dialog {
        id: up
        Button {
            text: i18n.tr("OK")
            color: UbuntuColors.green
            onClicked: {
                var upath = new Array()
                qo.x = 0
                up.hide()
                downloading.show()
                downloading.title = i18n.tr("Uploading...")
                files.visible = true
                for (var i=0; i<explorePage.activeTransfer.items.length; i++) {
                    upath.push(explorePage.activeTransfer.items[i].url.toString().replace("file://", ""))
                    downloading.tot = i + 1
                }
                py.call(ftptype + '.utils.upload', [upath, path, qo], function(result) {
                    downloading.hide()
                    files.visible = false
                    if(result === "error"){
                        error.show()
                        error.title = i18n.tr("Unable to upload the file")
                    }else{
                        explorePage.update()
                    }
                });
            }
        }
        Button {
            text: i18n.tr("Cancel")
            onClicked: up.hide()
        }
    }

    ContentPickerDialog {
        id: contentpop
    }
}
