import QtQuick 2.0
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import "Global.js" as Utils
import "Connect.js" as Connect

Page {
    id: page
    title: i18n.tr("Manage")

    head.actions: [
            Action {
                id: newAction
                text: i18n.tr("New server")
                iconName: "add"
                onTriggered: {
                    typeDialog.show()
                }
            },
            Action {
                id: downAction
                text: i18n.tr("Downloads")
                iconName: "save"
                onTriggered: {
                    pageStack.push(Qt.resolvedUrl("Downloads.qml"))
                }
            },
            Action {
                id: aboutAction
                text: i18n.tr("About")
                iconName: "info"
                onTriggered: {
                    pageStack.push(Qt.resolvedUrl("About.qml"))
                }
            }

        ]


       Label {
           id: none
           anchors.centerIn: parent
           text: i18n.tr("No server added.")
       }

        ServerListView {
                anchors.fill: parent
                model: servers
                delegate: ListItem {
                    Icon {
                        anchors {
                           left: parent.left
                           leftMargin: units.gu(3)
                           verticalCenter: parent.verticalCenter
                        }
                        name: "folder-remote-ftp"
                        height: units.gu(6)
                        width: units.gu(6)
                    }
                    Label {
                        anchors {
                           left: parent.left
                           right: parent.right
                           leftMargin: units.gu(10)
                           rightMargin: units.gu(5)
                           verticalCenter: parent.verticalCenter
                        }
                        elide: Text.ElideRight
                        text: model.contents.name
                        font.pixelSize: units.gu(2)
                        font.weight: Font.Light
                        horizontalAlignment: Text.AlignLeft
                    }
                    Icon {
                        anchors {
                            right: parent.right
                            rightMargin: units.gu(1)
                            verticalCenter: parent.verticalCenter
                        }
                        name: "lock"
                        visible: (model.contents.sftp === true)
                        height: units.gu(2)
                        width: units.gu(2)
                    }
                    onClicked: {
                        var serverObj = db.getDoc(model.docId)
                        root.address = serverObj.address
                        root.port = serverObj.port
                        if(serverObj.sftp && serverObj.pka)
                            root.anonym = "pka"
                        else
                            root.anonym = serverObj.anonym
                        root.user = serverObj.user
                        root.pwd = serverObj.pwd
                        root.mlsd = serverObj.mlsd
                        Connect.conn(serverObj.sftp, serverObj.mlsd)
                        if(serverObj.sftp)
                            pageStack.push(Qt.resolvedUrl("Explore.qml"), {exit: true, ftptype: "sftpmanager"})
                        else
                            pageStack.push(Qt.resolvedUrl("Explore.qml"), {exit: true, ftptype: "ftpmanager"})
                    }

                    leadingActions: ListItemActions {
                        actions: [
                            Action {
                                text: i18n.tr("Delete")
                                iconName: "delete"
                                onTriggered: {
                                    Utils.cached = model.docId
                                    dialog.show()
                                }
                            }
                        ]
                    }
                    trailingActions: ListItemActions {
                        actions: [
                            Action {
                                text: i18n.tr("Edit")
                                iconName: "edit"
                                onTriggered: {
                                    Utils.cached = model.docId
                                    pageStack.push(Qt.resolvedUrl("Edit.qml"))
                                }
                            }
                        ]
                    }
                    Component.onCompleted: none.visible = false
                    Component.onDestruction: none.visible = true
                }
            }

        Dialog {
            id: dialog
            title: i18n.tr("Delete server?")

            Button {
                text: i18n.tr("OK")
                color: UbuntuColors.orange
                onClicked: {
                    db.putDoc("", Utils.cached);
                    dialog.hide()
                }
            }

            Button {
                text: i18n.tr("Cancel")
                onClicked: dialog.hide()

            }
        }
        Dialog {
            id: typeDialog
            title: i18n.tr("Select the service")
            Button {
                text: "FTP"
                color: UbuntuColors.orange
                onClicked: {
                    pageStack.push(Qt.resolvedUrl("New.qml"), {sftp: false})
                    typeDialog.hide()
                }
            }

            Button {
                text: "SFTP"
                color: UbuntuColors.green
                onClicked: {
                    pageStack.push(Qt.resolvedUrl("New.qml"), {sftp: true})
                    typeDialog.hide()
                }

            }
        }

}
